@@ -3,7 +3,7 @@
 //! efficient numeric trait implementations, often times making use of
 //! SIMD instructions and compile-time evaluations.
 //!
-//! All stable `std::ops` traits are implemented for `NumericArray` itself,
+//! All stable `core::ops` traits are implemented for `NumericArray` itself,
 //! plus the thin `NumericConstant` type, which is required to
 //! differeniate constant values from `NumericArray` itself.
 //!
@@ -46,10 +46,9 @@
 //!
 
 #![deny(missing_docs)]
+#![no_std]
 
 extern crate num_traits;
-
-#[cfg(test)]
 extern crate typenum;
 
 #[cfg_attr(test, macro_use)]
@@ -58,15 +57,15 @@ extern crate generic_array;
 #[cfg(feature = "serde1")]
 extern crate serde;
 
-use std::{cmp, ptr, slice};
+use core::{cmp, ptr, slice};
 
-use std::borrow::{Borrow, BorrowMut};
-use std::ops::{Deref, DerefMut, Index, IndexMut};
-use std::ops::{Range, RangeFrom, RangeFull, RangeTo};
+use core::borrow::{Borrow, BorrowMut};
+use core::ops::{Deref, DerefMut, Index, IndexMut};
+use core::ops::{Range, RangeFrom, RangeFull, RangeTo};
 
-use std::iter::FromIterator;
+use core::iter::FromIterator;
 
-use std::fmt::{Debug, Formatter, Result as FmtResult};
+use core::fmt::{Debug, Formatter, Result as FmtResult};
 
 use generic_array::functional::*;
 use generic_array::sequence::*;
@@ -75,6 +74,7 @@ use generic_array::{ArrayLength, GenericArray, GenericArrayIter};
 #[cfg(feature = "serde1")]
 mod impl_serde;
 
+pub mod geometry;
 pub mod impls;
 pub mod simd;
 
@@ -86,7 +86,7 @@ pub mod simd;
 ///
 /// For example, adding together four-element `NumericArray`'s will result
 /// in a single SIMD instruction for all elements at once.
-#[repr(C)]
+#[repr(transparent)]
 pub struct NumericArray<T, N: ArrayLength<T>>(GenericArray<T, N>);
 
 /// Sugar for `NumericArray::new(arr![...])`
@@ -123,6 +123,7 @@ unsafe impl<T, N: ArrayLength<T>> GenericSequence<T> for NumericArray<T, N> {
 /// As a result, constants must be wrapped in this totally
 /// transparent wrapper type to differentiate the types to Rust.
 #[derive(Debug, Clone, Copy)]
+#[repr(transparent)]
 pub struct NumericConstant<T>(pub T);
 
 /// Creates a new `NumericConstant` from the given expression.
@@ -303,12 +304,12 @@ impl<T, N: ArrayLength<T>> NumericArray<T, N> {
     ///
     /// ```ignore
     /// let a = NumericArray::new(arr![i32; 5, 5, 5, 5]);
-    /// let b = NumericArray::from_element(5);
+    /// let b = NumericArray::splat(5);
     ///
     /// assert_eq!(a, b);
     /// ```
     #[inline]
-    pub fn from_element(t: T) -> NumericArray<T, N>
+    pub fn splat(t: T) -> NumericArray<T, N>
     where
         T: Clone,
     {
@@ -352,6 +353,71 @@ impl<T, N: ArrayLength<T>> NumericArray<T, N> {
     pub fn as_mut_slice(&mut self) -> &mut [T] {
         &mut self.0
     }
+
+    /// Converts slice to a numeric array reference with inferred length;
+    ///
+    /// Length of the slice must be equal to the length of the array.
+    #[inline]
+    pub fn from_slice(slice: &[T]) -> &NumericArray<T, N> {
+        slice.into()
+    }
+
+    /// Converts mutable slice to a mutable numeric array reference
+    ///
+    /// Length of the slice must be equal to the length of the array.
+    #[inline]
+    pub fn from_mut_slice(slice: &mut [T]) -> &mut NumericArray<T, N> {
+        slice.into()
+    }
+}
+
+use core::ops::Sub;
+use typenum::{bit::B1 as True, Diff, IsGreaterOrEqual};
+
+impl<T, N: ArrayLength<T>> NumericArray<T, N> {
+    /// Offset the numeric array and cast it into a shorter array
+    #[inline(always)]
+    pub fn offset<V: ArrayLength<T>, O: ArrayLength<T>>(&self) -> &NumericArray<T, V>
+    where
+        N: Sub<O>,
+        Diff<N, O>: IsGreaterOrEqual<V, Output = True>,
+    {
+        unsafe { &*((self as *const _ as *const T).add(O::USIZE) as *const NumericArray<T, V>) }
+    }
+
+    /// Offset the numeric array and cast it into a shorter array
+    #[inline(always)]
+    pub fn offset_mut<V: ArrayLength<T>, O: ArrayLength<T>>(&mut self) -> &mut NumericArray<T, V>
+    where
+        N: Sub<O>,
+        Diff<N, O>: IsGreaterOrEqual<V, Output = True>,
+    {
+        unsafe { &mut *((self as *mut _ as *mut T).add(O::USIZE) as *mut NumericArray<T, V>) }
+    }
+}
+
+impl<'a, T, N: ArrayLength<T>> From<&'a [T]> for &'a NumericArray<T, N> {
+    /// Converts slice to a numeric array reference with inferred length;
+    ///
+    /// Length of the slice must be equal to the length of the array.
+    #[inline]
+    fn from(slice: &[T]) -> &NumericArray<T, N> {
+        debug_assert_eq!(slice.len(), N::to_usize());
+
+        unsafe { &*(slice.as_ptr() as *const NumericArray<T, N>) }
+    }
+}
+
+impl<'a, T, N: ArrayLength<T>> From<&'a mut [T]> for &'a mut NumericArray<T, N> {
+    /// Converts mutable slice to a mutable numeric array reference
+    ///
+    /// Length of the slice must be equal to the length of the array.
+    #[inline]
+    fn from(slice: &mut [T]) -> &mut NumericArray<T, N> {
+        debug_assert_eq!(slice.len(), N::to_usize());
+
+        unsafe { &mut *(slice.as_mut_ptr() as *mut NumericArray<T, N>) }
+    }
 }
 
 impl<T, N: ArrayLength<T>> AsRef<[T]> for NumericArray<T, N> {
@@ -381,50 +447,50 @@ impl<T, N: ArrayLength<T>> BorrowMut<[T]> for NumericArray<T, N> {
 impl<T, N: ArrayLength<T>> Index<usize> for NumericArray<T, N> {
     type Output = T;
 
-    #[inline]
+    #[inline(always)]
     fn index(&self, index: usize) -> &T {
-        &(**self)[index]
+        self.0.index(index)
     }
 }
 
 impl<T, N: ArrayLength<T>> IndexMut<usize> for NumericArray<T, N> {
-    #[inline]
+    #[inline(always)]
     fn index_mut(&mut self, index: usize) -> &mut T {
-        &mut (**self)[index]
+        self.0.index_mut(index)
     }
 }
 
 impl<T, N: ArrayLength<T>> Index<Range<usize>> for NumericArray<T, N> {
     type Output = [T];
 
-    #[inline]
+    #[inline(always)]
     fn index(&self, index: Range<usize>) -> &[T] {
-        Index::index(&**self, index)
+        self.0.index(index)
     }
 }
 
 impl<T, N: ArrayLength<T>> Index<RangeTo<usize>> for NumericArray<T, N> {
     type Output = [T];
 
-    #[inline]
+    #[inline(always)]
     fn index(&self, index: RangeTo<usize>) -> &[T] {
-        Index::index(&**self, index)
+        self.0.index(index)
     }
 }
 
 impl<T, N: ArrayLength<T>> Index<RangeFrom<usize>> for NumericArray<T, N> {
     type Output = [T];
 
-    #[inline]
+    #[inline(always)]
     fn index(&self, index: RangeFrom<usize>) -> &[T] {
-        Index::index(&**self, index)
+        self.0.index(index)
     }
 }
 
 impl<T, N: ArrayLength<T>> Index<RangeFull> for NumericArray<T, N> {
     type Output = [T];
 
-    #[inline]
+    #[inline(always)]
     fn index(&self, _index: RangeFull) -> &[T] {
         self
     }
@@ -482,7 +548,7 @@ pub mod tests {
     // This stops the compiler from optimizing based on known data, only data types.
     #[inline(never)]
     pub fn black_box<T>(val: T) -> T {
-        use std::{mem, ptr};
+        use core::{mem, ptr};
 
         let ret = unsafe { ptr::read_volatile(&val) };
         mem::forget(val);
@@ -548,7 +614,7 @@ pub mod tests {
     #[test]
     fn test_classify() {
         use num_traits::Float;
-        use std::num::FpCategory;
+        use core::num::FpCategory;
 
         let nan = f32::nan();
         let infinity = f32::infinity();
@@ -582,9 +648,7 @@ pub mod tests {
 
         let a = black_box(narr![f32; 1, 2, 3, 4]);
 
-        let b = a.tanh();
-
-        println!("{:?}", b);
+        black_box(a.tanh());
     }
 
     #[test]
